@@ -3,11 +3,12 @@ our $AUTHORITY = 'cpan:GENE';
 
 # ABSTRACT: Partition a musical duration
 
-our $VERSION = '0.0401';
+our $VERSION = '0.0500';
 
 use Moo;
 use strictures 2;
 
+use Math::Random::Discrete;
 use MIDI::Simple;
 use List::Util qw/ min /;
 
@@ -32,18 +33,6 @@ has pool => (
     default => sub { return [ keys %MIDI::Simple::Length ] },
 );
 
-
-has pool_code => (
-    is      => 'rw',
-    builder => 1,
-    lazy    => 1,
-);
-
-sub _build_pool_code {
-    my ($self) = @_;
-    return sub { return $self->pool->[ int rand @{ $self->pool } ] };
-};
-
 has _min_size => (
     is       => 'ro',
     builder  => 1,
@@ -60,6 +49,32 @@ sub _build__min_size {
 }
 
 
+has pool_code => (
+    is      => 'rw',
+    builder => 1,
+    lazy    => 1,
+);
+
+sub _build_pool_code {
+    my ($self) = @_;
+    my $choice = Math::Random::Discrete->new($self->weights, $self->pool);
+    return sub { return $choice->rand };
+};
+
+
+has weights => (
+    is      => 'ro',
+    builder => 1,
+    lazy    => 1,
+);
+
+sub _build_weights {
+    my ($self) = @_;
+    # Equal probability for all pool members
+    return [ map { 1 } @{ $self->pool } ];
+}
+
+
 has verbose => (
     is      => 'ro',
     default => sub { return 0 },
@@ -124,7 +139,7 @@ Music::Duration::Partition - Partition a musical duration
 
 =head1 VERSION
 
-version 0.0401
+version 0.0500
 
 =head1 SYNOPSIS
 
@@ -133,8 +148,9 @@ version 0.0401
   use Music::Scales;
 
   my $mdp = Music::Duration::Partition->new(
-    size => 8,
-    pool => [qw/ qn en sn /],
+    size    => 8,
+    pool    => [qw/ qn en sn /],
+    weights => [ 0.2, 0.3, 0.5 ],
   );
 
   $mdp->pool_code( sub { ... } ); # Optional
@@ -203,6 +219,14 @@ B<pool>.
 
 Default: Random item of B<pool>
 
+=head2 weights
+
+  $weights = $mdp->weights;
+
+Specification of the frequency of pool selection.
+
+Default: Equal probability for each pool entry
+
 =head2 verbose
 
   $verbose = $mdp->verbose;
